@@ -23,6 +23,7 @@
 import argparse
 import ast
 import copy
+import errno
 import os
 import re
 import signal
@@ -31,8 +32,9 @@ import sys
 import tempfile
 import textwrap
 import time
+from typing import List
 
-__version__ = '1.1.1'
+__version__ = '1.2.0'
 __docformat__ = "restructuredtext"
 
 # format settings
@@ -42,6 +44,7 @@ FORMAT = 'format'
 VIEWER = 'viewer'
 OUT_FILE = 'outfile'
 WAIT_SECONDS = 'wait'
+SIMPLIFY = 'simplify'
 OUTPUT_SETTINGS = [
     FORMAT,
     GRAPHVIZ,
@@ -49,6 +52,7 @@ OUTPUT_SETTINGS = [
     VIEWER,
     OUT_FILE,
     WAIT_SECONDS,
+    SIMPLIFY,
 ]
 OUTPUT_DEFAULTS = {
     FORMAT: 'svg',
@@ -57,6 +61,7 @@ OUTPUT_DEFAULTS = {
     VIEWER: False,
     OUT_FILE: False,
     WAIT_SECONDS: 2.0,
+    SIMPLIFY: False,
 }
 
 # filter settings
@@ -100,6 +105,9 @@ EXIT_CODES = {
     "no_options": 8,
     "no_git": 9,
     "no_git_repo": 10,
+    "tred_not_found": 11,
+    "problem_with_tred": 12,
+    "tred_terminated_early": 13,
     "killed_by_sigint": 128 + signal.SIGINT,
 }
 
@@ -145,10 +153,15 @@ def create_parser():
                               help='set output format [svg, png, ps, pdf, ...]')
 
     format_group.add_argument('--history-direction',
-                              default=None,
+                              default='rightwards',
                               choices=sorted(RANKDIR_OF_HISTORY_DIRECTION.keys()),
                               help='enforce a specific direction of history on Graphviz\n'
-                              '(default: upwards)')
+                              '(default: %(default)s)')
+
+    format_group.add_argument('--simplify',
+                              action='store_true',
+                              help='remove edges implied by transitivity using Graphviz\n'
+                              'filter "tred" (default: do not remove implied edges)')
 
     format_group.add_argument('-g',
                               '--graphviz',
@@ -397,6 +410,36 @@ def parse_variable_args(args):
     return args[0] if len(args) == 1 else os.getcwd()
 
 
+def run_graphviz_command(argv: List[str],
+                         stdin_lines: List[str],
+                         enoent_exit_code: int,
+                         nonzero_exit_code: int,
+                         exception_exit_code: int,
+                         hint: str = ''):
+    tool = argv[0]
+    try:
+        p = subprocess.Popen(argv,
+                             stdin=subprocess.PIPE,
+                             stdout=subprocess.PIPE,
+                             stderr=subprocess.PIPE)
+    except OSError as e:
+        if e.errno == errno.ENOENT:
+            barf(f'{tool!r} not found! Please install the Graphviz utility.', enoent_exit_code)
+        else:
+            barf(f'A problem occured calling {" ".join(argv)!r}', exception_exit_code)
+
+    out, err = p.communicate(input='\n'.join(stdin_lines).encode('utf-8'))
+
+    if p.returncode != 0:
+        hint_part = f';\n{hint}' if hint else ''
+        barf(
+            f'{tool!r} terminated prematurely with error code {p.returncode}{hint_part}.\n'
+            f'The error from {tool!r} was:\n'
+            f'>>>{err.decode("utf-8")}', nonzero_exit_code)
+
+    return out
+
+
 def run_dot(output_format, dot_file_lines):
     """ Run the 'dot' utility.
 
@@ -412,28 +455,35 @@ def run_dot(output_format, dot_file_lines):
     Raw output from 'dot' utility
 
     """
-    try:
-        p = subprocess.Popen(['dot', '-T' + output_format],
-                             stdin=subprocess.PIPE,
-                             stdout=subprocess.PIPE,
-                             stderr=subprocess.PIPE)
-    except OSError as e:
-        if e.errno == 2:
-            barf("'dot' not found! Please install the Graphviz utility.",
-                 EXIT_CODES["dot_not_found"])
-        else:
-            barf("A problem occured calling 'dot -T%s'" % output_format,
-                 EXIT_CODES["problem_with_dot"])
+    return run_graphviz_command(
+        argv=['dot', f'-T{output_format}'],
+        stdin_lines=dot_file_lines,
+        enoent_exit_code=EXIT_CODES['dot_not_found'],
+        nonzero_exit_code=EXIT_CODES['dot_terminated_early'],
+        exception_exit_code=EXIT_CODES['problem_with_dot'],
+        hint='probably you specified an invalid format, see \'man dot\'',
+    )
 
-    # send dot input, automatically receive and store output and error
-    out, err = p.communicate(input='\n'.join(dot_file_lines).encode('utf-8'))
-    if p.returncode != 0:
-        barf(
-            "'dot' terminated prematurely with error code %d;\n"
-            "probably you specified an invalid format, see 'man dot'.\n"
-            "The error from 'dot' was:\n"
-            ">>>%s" % (p.returncode, err.decode('utf-8')), EXIT_CODES["dot_terminated_early"])
-    return out
+
+def simplify_using_tred(dot_file_lines):
+    """ Run the 'tred' utility.
+
+    Parameters
+    ----------
+    dot_file_lines : list of strings
+        graphviz input lines
+
+    Returns
+    -------
+    Raw output from 'tred' utility
+    """
+    return run_graphviz_command(
+        argv=['tred'],
+        stdin_lines=dot_file_lines,
+        enoent_exit_code=EXIT_CODES['tred_not_found'],
+        nonzero_exit_code=EXIT_CODES['tred_terminated_early'],
+        exception_exit_code=EXIT_CODES['problem_with_tred'],
+    )
 
 
 def write_to_file(output_file, dot_output):
@@ -606,7 +656,10 @@ def get_command_output(command_list, cwd=None, git_env=None):
     p.stderr.close()
     p.wait()
     if p.returncode:
-        err = p.stderr.read()
+        try:
+            err = p.stderr.read()
+        except ValueError:  # case "read of closed file"
+            err = ''
         err = '\n'.join(('> ' + e) for e in err.split('\n'))
         raise Exception('Stderr:\n%s\nReturn code %d from command "%s"' %
                         (err, p.returncode, ' '.join(command_list)))
@@ -614,6 +667,7 @@ def get_command_output(command_list, cwd=None, git_env=None):
 
 
 class Git:
+
     def __init__(self, repo_dir):
 
         self.repo_dir = repo_dir
@@ -777,6 +831,7 @@ class CommitGraph:
         interface to dispatch commands to this repo
 
     """
+
     def __init__(self, parent_map, branch_dict, tag_dict, git=None):
         self.parents = parent_map
         self.branches = branch_dict
@@ -947,6 +1002,7 @@ class CommitGraph:
         dot_file_lines : list of strings
             lines of the graphviz input
         """
+
         def format_sha_one(sha_one):
             """ Shorten sha1 if required. """
             if (sha_one_digits is None) or (sha_one_digits == 40):
@@ -968,10 +1024,10 @@ class CommitGraph:
                 labels = []
                 case = 0
                 if k in self.tags:
-                    case = case + 1
+                    case += 1
                     labels.extend(sorted(self.tags[k]))
                 if k in self.branches:
-                    case = case + 2
+                    case += 2
                     labels.extend(sorted(self.branches[k]))
                 # http://www.graphviz.org/doc/info/colors.html
                 color = "/pastel13/%d" % case
@@ -1043,6 +1099,10 @@ def innermost_main(opts):
     ]):
         barf("Must provide an output option. Try '-h' for more information",
              EXIT_CODES["no_options"])
+
+    if output_settings[SIMPLIFY]:
+        dot_file_lines = simplify_using_tred(dot_file_lines).decode('utf-8').split('\n')
+
     # if plain just print dot input to stdout
     if output_settings[GRAPHVIZ]:
         debug('Will now print dot format')
